/*--------------------------------------------------------------------

  World Folio - Offline map viewer/editor for Age of Adventure
  
  Copyright (c) 2001 Phil Dempster <phil@buxton.port5.com>
  http://buxton.port5.com/age/

  This software is based on an original work the copyright in which
  vests in Interactive Broadcasting Ltd (http://www.ibgames.net/).
  Original programming by Alan Lenton with additional programming by
  Stanley Lenton.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Compiled with Borland C++ Builder 3.
    
--------------------------------------------------------------------*/

#ifndef LOCATION_H
#define LOCATION_H

#include <cstdio>

using namespace std;

class Location
{
public:
    enum    { MAX_NAME = 40 };

    static int mincol;
    static int maxcol;
    static int minrow;
    static int maxrow;

protected:
    int column;                 // x position on the map
    int row;                    // y position on the map
    int exits;                  // Normal exits from this location
    char    name[MAX_NAME];     // The location's name

    char    *notes;             // Player's own notes, if any

    int terrain;                // Special locations.
    int special;                // Special locations.
    int user;                   // Special locations.

    int CheckAutomagically(const char *p);
    int CheckSpecialLoc(const char *p);

public:
    Location(char *txt);
    ~Location();

    int Column()                    { return(column);   }
    int Row()                       { return(row);      }
    int Exits()                     { return(exits);    }
    const char  *Notes()            { return(notes);    }
    const char  *Name()             { return(name);     }
    int          Special()          { return(user>-1 ? user:special); }
    void         Special(int s)     { special = s;      }
    void         User(int u)        { user    = u;      }

    bool    IsOK()                  { return(column != -1); }

    void    Save(FILE *file);
    void    ShowNotes(void);
    void    UpdateNotes(void);

    static  void GetCentre(int &ctrcol, int &ctrrow);
    static  void ResetCentre(void);
};

#endif


// Location properties.
#define I_NOTES     1
#define I_DEFAULT   2
#define I_SPECIAL   8
#define I_USER      (I_SPECIAL+10)

// Fixed properties.
#define I_UNLIT     (I_SPECIAL)
#define I_PAWNSHOP  (I_SPECIAL+1)
#define I_CAFEBAR   (I_SPECIAL+2)
#define I_RAILWAY   (I_SPECIAL+3)
#define I_PORT      (I_SPECIAL+4)
#define I_AIRSHIP   (I_SPECIAL+5)
#define I_WATER     (I_SPECIAL+6)
#define I_INSTITUTE (I_SPECIAL+7)
#define I_HOSPITAL  (I_SPECIAL+8)
#define I_TELEGRAPH (I_SPECIAL+9)

// User properties.
#define I_RATIONS   (I_USER)
#define I_WEAPON    (I_USER+1)
#define I_SKILL     (I_USER+2)

// Temp (GUI only) property.
#define I_CROSS     (I_USER+3)
