/*--------------------------------------------------------------------

  World Folio - Offline map viewer/editor for Age of Adventure

  Copyright (c) 2001 Phil Dempster <phil@buxton.port5.com>
  http://buxton.port5.com/age/

  This software is based on an original work the copyright in which
  vests in Interactive Broadcasting Ltd (http://www.ibgames.net/).
  Original programming by Alan Lenton with additional programming by
  Stanley Lenton.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Compiled with Borland C++ Builder 3.

--------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "main.h"

#include <cstdio>
#include <cstring>
#include <cdir>

using namespace std;

#include "age_map.h"
#include "location.h"
#include "folio.h"
#include "index.h"
#include "notes.h"
#include "about.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------

__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
    is_clean = true;
    strcpy(version, "Wayfarer's Edition");
}

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    if(!is_clean)
    {
        switch(MessageDlg("You have unsaved notes -\nsave map to disk?",mtConfirmation,TMsgDlgButtons() << mbYes << mbNo << mbCancel,0))
        {
            case mrYes: SaveItemClick(this);    Action = caFree;    break;
            case mrNo:  Action = caFree;                                break;
            case mrCancel:  Action = caNone;                            return;
        }
    }

    delete age_map;
    AgeMap::DeleteMapGfx();

    return;
}

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
    char    buffer[160];

    age_map = 0;
    AgeMap::LoadMapGfx();
    file_name[0] = '\0';
    sprintf(buffer, "%s - %s", Caption.c_str(), version);
    Caption = buffer;
}



void __fastcall TMainForm::ExitItemClick(TObject *Sender)
{
    Close();
}

void __fastcall TMainForm::GridDrawCell(TObject *Sender, int ACol,
        int ARow, TRect &Rect, TGridDrawState State)
{
    if(age_map != 0)
        age_map->DrawCell(ACol,ARow,Rect,Grid->DefaultColWidth);
    return;
}

void __fastcall TMainForm::GridMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
    int cur_col,cur_row;

    Grid->MouseToCell(X,Y,cur_col,cur_row);
    ShowNotes(cur_col,cur_row,X,Y);
    return;
}

void __fastcall TMainForm::GridMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
    int cur_col,cur_row;

    Grid->MouseToCell(X,Y,cur_col,cur_row);
    if(age_map != 0)
        age_map->Name2Status(cur_col,cur_row);

    return;
}


bool TMainForm::LoadMap(char *fn)
{
    FILE *file;

    if (NotesForm->Visible)
        NotesForm->Close();         //FIXME inelegant.
    if (age_map)
    {
        if(!is_clean)
        {
            switch(MessageDlg("You have unsaved notes -\nsave map to disk?",mtConfirmation,TMsgDlgButtons() << mbYes << mbNo << mbCancel,0))
            {
                case mrYes:
                    SaveItemClick(this);
                    // Fall thru'.
                case mrNo:
                    break;
                case mrCancel:
                    return false;
            }
        }
        delete age_map;
    }

    strcpy(file_name, fn);

    if((file = fopen(file_name,"r")) != 0)
    {
        Grid->Enabled  = false;
        Location::ResetCentre();
        age_map = new AgeMap(file);
        fclose(file);

        SaveItem->Enabled = (age_map->filetype == AgeMap::MAP) ? true : false;
        is_clean = true;

        Grid->ColCount = (Location::maxcol - Location::mincol) * 2 + 2;
        Grid->RowCount = (Location::maxrow - Location::minrow) * 2 + 2;
        Grid->Enabled  = true;
        Grid->Refresh();

        FindNames();
        return true;
    }
    else
    {
        MessageDlg("Can't open file: "+AnsiString(file_name), mtInformation, TMsgDlgButtons() << mbOK, 0);
        return false;
    }
}


void __fastcall TMainForm::OpenItemClick(TObject *Sender)
{
    if(OpenDialog->Execute())
    {
        if (LoadMap(OpenDialog->FileName.c_str()))
        {
            FolioForm->ReadFolio(OpenDialog->InitialDir.c_str(), "*.map");
        }
    }
}


void __fastcall TMainForm::SaveItemClick(TObject *Sender)
{
    FILE    *file;

    if((file = fopen(file_name,"w")) == 0)
        MessageDlg("Can't open file!",mtInformation,TMsgDlgButtons() << mbOK,0);
    else
    {
        age_map->Save(file);
        fclose(file);
        MessageDlg("Map Saved",mtInformation,TMsgDlgButtons() << mbOK,0);
        is_clean = true;
    }

    return;
}


void TMainForm::CentreMap(int grid_col, int grid_row)
{
    // Centre map on selected location.

    grid_col -= Grid->VisibleColCount / 2;

    if ((grid_col + Grid->VisibleColCount) >= Grid->ColCount)
        grid_col = Grid->ColCount - Grid->VisibleColCount;

    if (grid_col < 0)
        grid_col = 0;

    grid_row -= Grid->VisibleRowCount / 2;

    if ((grid_row + Grid->VisibleRowCount) >= Grid->RowCount)
        grid_row = Grid->RowCount - Grid->VisibleRowCount;

    if (grid_row < 0)
        grid_row = 0;

    Grid->Enabled = false;
    Grid->LeftCol = grid_col;
    Grid->TopRow  = grid_row;
    Grid->Enabled = true;
    Grid->Refresh();
}

//---------------------------------------------------------------------------
void TMainForm::FindNames(void)
{
     if (age_map)
        age_map->FindNames();
}
void __fastcall TMainForm::FolioItemClick(TObject *Sender)
{
    if (!FolioForm->Visible)
    {
        FolioForm->Show();
    }
    else
    {
        FolioForm->Hide();
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::IndexItemClick(TObject *Sender)
{
    if (!IndexForm->Visible)
    {
        IndexForm->Show();
    }
    else
    {
        IndexForm->Hide();
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::AboutItemClick(TObject *Sender)
{
    AboutForm->Show();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ZoomInItemClick(TObject *Sender)
{
    int tile_size = (Grid->DefaultColWidth * 3)/2;

    age_map->Zoom(false);

    ZoomInItem->Enabled     = false;
    ZoomOutItem->Enabled    = true;

    Grid->Enabled = false;
    Grid->DefaultColWidth   = tile_size;
    Grid->DefaultRowHeight  = tile_size;
    Grid->Enabled = true;
    Grid->Refresh();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ZoomOutItemClick(TObject *Sender)
{
    int tile_size = (Grid->DefaultColWidth * 2)/3;

    age_map->Zoom(true);

    ZoomInItem->Enabled     = true;
    ZoomOutItem->Enabled    = false;

    Grid->Enabled = false;
    Grid->DefaultColWidth   = tile_size;
    Grid->DefaultRowHeight  = tile_size;
    Grid->Enabled = true;
    Grid->Refresh();
}
//---------------------------------------------------------------------------


inline void TMainForm::ShowNotes(int col, int row, int X, int Y)
{
    if (age_map != 0)
    {
        age_map->ShowNotes(col, row, X, Y);
    }
}


