// pngdib.h


// error codes returned by read_png_to_dib() and/or
//   write_dib_to_png()

#define PNGD_E_SUCCESS   0
#define PNGD_E_ERROR     1   // unspecified error
#define PNGD_E_VERSION   2   // struct size problem
#define PNGD_E_NOMEM     3   // could not alloc memory
#define PNGD_E_UNSUPP    4   // unsupported image type
#define PNGD_E_LIBPNG    5   // libpng error (corrupt PNG?)
#define PNGD_E_BADBMP    6   // corrupt or unsupported DIB
#define PNGD_E_BADPNG    7   // corrupt or unsupported PNG
#define PNGD_E_READ      8   // couldn't read PNG file
#define PNGD_E_WRITE     9   // couldn't write PNG file


typedef struct PNGD_D2PINFO_struct {
	DWORD           structsize;      // sizeof(PNGD_D2PINFO)
	DWORD           flags;
#define PNGD_INTERLACED  0x00000001

	char*           pngfn;          // PNG filename to write

	LPBITMAPINFOHEADER    lpdib;
	int             dibsize;        // can be 0

	VOID*           lpbits;         // can be NULL
	int             bitssize;       // can be 0

	char*           software;       // (NULL==don't include)
} PNGD_D2PINFO;


typedef struct PNGD_P2DINFO_struct {
	DWORD           structsize;      // sizeof(PNGD_P2DINFO)
	DWORD           flags;           // reserved, set to 0
	char*           pngfn;           // PNG filename to read

	LPBITMAPINFOHEADER    lpdib;           // return value only
	int             dibsize;         // return value only
	int             palette_offs;    // return value only
	int             bits_offs;       // return value only
	RGBQUAD*        palette;         // return value only
	int             palette_colors;  // return value only
	VOID*           bits;            // return value only
} PNGD_P2DINFO;


#ifdef __cplusplus
extern "C"
{
#endif
int read_png_to_dib(PNGD_P2DINFO *p2dinfo);

int write_dib_to_png(PNGD_D2PINFO *d2pinfo);
#ifdef __cplusplus
}
#endif

